using Apaya.Connector.Template.Models.Charge;
using Apaya.Connector.Template.Models.CheckStatus;
using Apaya.Connector.Template.Models.Refunds;
using Apaya.Connector.Template.Models.Tokens;
using Apaya.Connector.Template.Models.Void;
using Apaya.Connector.Core.Actions;
using Apaya.Connector.Core.Actions.CapturePayment;
using Apaya.Connector.Core.Actions.GetTransaction;
using Apaya.Connector.Core.Actions.ProcessPayment;
using Apaya.Connector.Core.Actions.ProcessRefund;
using Apaya.Connector.Core.Actions.VoidPayment;
using System;

namespace Apaya.Connector.Template.Helpers
{
    public static class Mapper
    {
        //Functions here map a connector request/response payload to our generic request/response payloads
        public static CardTokenRequest MapSinglePayment(this ProcessPaymentRequest request)
        {
            return new CardTokenRequest
            {
                client_ip = request.Order?.Customer?.IpAddress,
                card = new CardTokenRequestObject()
                {
                    address = new Models.Address(),
                    cvc = Convert.ToInt32(request.Payment.Card.CVV),
                    exp_month = request.Payment.Card.ExpiryMonth,
                    exp_year = request.Payment.Card.ExpiryYear,
                    name = request.Payment.Card.CardHolder,
                    number = Convert.ToInt64(request.Payment.Card.CardNumber)
                }
            };
        }

        public static SavedCardTokenRequest MapRecurringPayment(this ProcessPaymentRequest request)
        {
            //the previous payment id will be in this format: CARD_ID&&CUS_ID
            var splittedValue = request.Payment.PreviousPaymentId.Split("&&");

            return new SavedCardTokenRequest
            {
                client_ip = request.Order?.Customer?.IpAddress,
                saved_card = new Saved_Card()
                {
                    card_id = splittedValue[0],
                    customer_id = splittedValue[1]
                }
            };
        }



        public static ChargeRequest MapCharge(this ProcessPaymentRequest request, CardTokenResponse cardToken, bool isRecurring = true)
        {
            string idValue = isRecurring ? request?.Payment?.PreviousPaymentId?.Split("&&")[1] : null;

            return new ChargeRequest
            {
                Amount = Convert.ToInt32(request.Order.TotalAmount),
                Currency = request.Order.Currency,
                ThreeDSecure = !isRecurring,
                Save_card = !isRecurring,
                Description = request.Order.Description,
                Customer = new Models.Customer
                {
                    email = isRecurring ? null : request.Order.Customer.Email,
                    first_name = isRecurring ? null : request.Order.Customer.FirstName,
                    last_name = isRecurring ? null : request.Order.Customer.LastName,
                    id = idValue
                },
                Reference = new Models.Reference
                {
                    transaction = request.Payment.ApayaTransactionId.ToString(),
                    order = request.Payment.PassThrough
                },
                Source = new Models.Source
                {
                    Id = cardToken.id
                },
                Redirect = new Models.Redirect
                {
                    Url = request.SuccessUrl
                },
                Post = new Models.Post
                {
                    Url = Environment.GetWebhookEnvironmentUrl()
                }
            };
        }

        public static ChargeRequest MapCapture(this CapturePaymentRequest request)
        {
            return new ChargeRequest
            {
                Amount = Convert.ToInt32(request.Amount),
                Currency = request.Currency,
                //Customer = new Models.Customer
                //{
                //    id = idValue
                //},
                Reference = new Models.Reference
                {
                    transaction = request.Reference,
                },
                Source = new Models.Source
                {
                    Id = request.TransactionId
                },
                Post = new Models.Post
                {
                    Url = Environment.GetWebhookEnvironmentUrl()
                }
            };
        }

        public static CapturePaymentResponse MapCaptureResponse(this ChargeResponse response)
        {
            return new CapturePaymentResponse
            {
                Content = response,
                ActionStatus = new Status
                {
                    Code = response.response.code,
                    Message = response.response.message
                },
                ExternalPaymentId = response.id
            };
        }


        public static ProcessPaymentResponse Map(this ChargeResponse response)
        {
            return new ProcessPaymentResponse
            {
                Content = response,
                ActionStatus = new Status
                {
                    Code = response.response.code,
                    Message = response.response.message
                },
                NextActions = new NextActions
                {
                    ThreeDS = new ThreeDS
                    {
                        Url = response.transaction.url,
                        Required = response.threeDSecure
                    }
                },
                ExternalPaymentId = response.id,
                ExternalTokenReference = response.customer?.id
            };
        }


        public static GetTransactionResponse Map(this CheckStatusResponse response)
        {
            return new GetTransactionResponse
            {
                Id = response.reference.transaction,
                TransactionStatus = new Status
                {
                    Code = response.response.code,
                    Message = response.response.message
                }
            };
        }

        public static RefundRequest Map(this ProcessRefundRequest request)
        {
            return new RefundRequest
            {
                Amount = request.Amount.Value,
                ChargeId = request.ConnectorTransactionId,
                Currency = request.Currency,
                Reason = request.Reason,
                Post = new Models.Post
                {
                    Url = Environment.GetWebhookEnvironmentUrl()
                }
            };
        }

        public static ProcessRefundResponse Map(this RefundResponse response)
        {
            return new ProcessRefundResponse
            {
                Content = response,
                ActionStatus = new Status
                {
                    Code = response.Code,
                    Message = response.Message
                }
            };
        }

        public static VoidRequest Map(this VoidPaymentRequest request)
        {
            return new VoidRequest
            {
                ChargeId = request.TransactionId,
                Post = new Models.Post
                {
                    Url = Environment.GetWebhookEnvironmentUrl()
                }
            };
        }

        public static VoidPaymentResponse Map(this VoidResponse response)
        {
            return new VoidPaymentResponse
            {
                Content = response,
                ActionStatus = new Status
                {
                    Code = response.Code,
                    Message = response.Message
                }
            };
        }
    }
}