using System;
using Apaya.Connector.Template.Helpers;
using Apaya.Connector.Template.Models.Charge;
using Apaya.Connector.Template.Models.Common;
using Apaya.Connector.Template.Models.Configuration;
using Apaya.Connector.Template.Models.Tokens;
using Apaya.Connector.Core.Actions;
using Apaya.Connector.Core.Actions.ProcessPayment;
using Apaya.Connector.Core.Configuration;
using Apaya.Connector.Core.Exceptions;
using Newtonsoft.Json;
using RestSharp;


namespace Apaya.Connector.Template
{
    //Example Payment Action Handler  for single or recurring based off Core Connector dll
    public class ProcessPaymentAction : IProcessPaymentAction
    {
        private static string _connectorName;
        private static string _secretKey;

        private RestClient connRestClient { get; set; }
        private TemplateConfiguration _connectorConfig { get; set; }

        public ProcessPaymentAction()
        {
            _connectorName = Constants.ConnectorName;
            _secretKey = Constants.SecretKey;

        }

        public async Task<ProcessPaymentResponse> ProcessPaymentAsync(ProcessPaymentRequest request,
            AvailableConnectorConfiguration configuration,
            CancellationToken cancellationToken)
        {
            // Validate the configuration holds the required keys for this connector - _secretKey in this case
            DefaultConnectorConfigurationValidator validator = new DefaultConnectorConfigurationValidator();
            validator.ValidateMerchantConfig(_connectorName, configuration, new[] { _secretKey });

            _connectorConfig = new TemplateConfiguration
            {
                SecretKey = configuration.MerchantConfiguration![_secretKey],
            };

            // Setting the payment type single/recurrent based off passed in TransactionType
            ProcessPaymentResponse response = new ProcessPaymentResponse();
            switch (request.Payment!.TransactionType)
            {
                case TransactionType.Recurrent:
                case TransactionType.RecurringReserved:
                    response = await ExecuteRecurrentPaymentAsync(request, _connectorConfig, cancellationToken);
                    break;
                case TransactionType.Single:
                case TransactionType.SingleReserved:
                    response = await ExecuteSinglePaymentAsync(request, _connectorConfig, cancellationToken);
                    break;
            }

            //captured for logging purposes
            response.RawActionRequest = new RawConnectorPayload(ConnectorPayloadContentType.Json,
                new string[] { }, JsonConvert.SerializeObject(request));
            response.RawActionResponse = new RawConnectorPayload(ConnectorPayloadContentType.Json,
                new string[] { }, JsonConvert.SerializeObject(response.Content));

            return response;
        }

        private async Task<ProcessPaymentResponse> ExecuteRecurrentPaymentAsync(ProcessPaymentRequest request,
            TemplateConfiguration configuration, CancellationToken cancellationToken)
        {
            try
            {
                //Mapping generic request 
                var command = request.MapRecurringPayment();

                // Example - setup headers for connector API call
                var connRequest = new RestRequest(Method.POST);
                connRequest.AddHeader("content-type", Constants.JsonType);
                connRequest.AddHeader("Authorization", $"Bearer {configuration.SecretKey}");

                var settings = new JsonSerializerSettings()
                {
                    ContractResolver = new OrderedContractResolver(),
                    DefaultValueHandling = DefaultValueHandling.Ignore,
                    NullValueHandling = NullValueHandling.Ignore
                };

                // Example - Build json payload and Make API call
                connRestClient = new RestClient(Helpers.Environment.GetEnvironmentUrl(TokenUrl.BaseProductionUrl, TokenUrl.BaseSandboxUrl));

                var json = JsonConvert.SerializeObject(command, Formatting.Indented, settings);
                connRequest.AddParameter(Constants.JsonType, json, ParameterType.RequestBody);

                var connResponse = await connRestClient.ExecuteAsync(connRequest, cancellationToken);

                //Example - Handle response
                CardTokenResponse resp =
                    JsonConvert.DeserializeObject<CardTokenResponse>(connResponse.Content);

                var chargeResponse = await ChargeAsync(resp, request, configuration, new CancellationToken(), true);

                // see Mapper class
                return chargeResponse.Map();
            }
            catch (Exception e)
            {
                // may want to handle specific errors
                throw new GenericConnectorException(e.Message, e.Data.ToString());
            }
        }


        private async Task<ProcessPaymentResponse> ExecuteSinglePaymentAsync(ProcessPaymentRequest request,
            TemplateConfiguration configuration, CancellationToken cancellationToken)
        {
            try
            {
                //Mapping generic request 
                var command = request.MapSinglePayment();

                // Example - setup headers for connector API call
                var connRequest = new RestRequest(Method.POST);
                connRequest.AddHeader("content-type", Constants.JsonType);
                connRequest.AddHeader("Authorization", $"Bearer {configuration.SecretKey}");

                var settings = new JsonSerializerSettings()
                {
                    ContractResolver = new OrderedContractResolver(),
                    //DefaultValueHandling = DefaultValueHandling.Ignore,
                    NullValueHandling = NullValueHandling.Ignore
                };

                // Example - Build json payload and Make API call
                connRestClient = new RestClient(Helpers.Environment.GetEnvironmentUrl(TokenUrl.BaseProductionUrl, TokenUrl.BaseSandboxUrl));

                var json = JsonConvert.SerializeObject(command, Formatting.Indented, settings);
                connRequest.AddParameter(Constants.JsonType, json, ParameterType.RequestBody);

                var connResponse = await connRestClient.ExecuteAsync(connRequest, cancellationToken);

                //Example - Handle response
                CardTokenResponse resp =
                    JsonConvert.DeserializeObject<CardTokenResponse>(connResponse.Content);

                var chargeResponse = await ChargeAsync(resp, request, configuration, new CancellationToken());

                // see Mapper class
                return chargeResponse.Map();
            }
            catch (Exception e)
            {
                // may want to handle specific errors
                throw new GenericConnectorException(e.Message, "500");
            }
        }

        public async Task<ChargeResponse> ChargeAsync(CardTokenResponse cardToken, ProcessPaymentRequest request,
            TemplateConfiguration configuration, CancellationToken cancellationToken, bool isRecurring = false)
        {
            try
            {
                //Mapping generic request 
                var command = request.MapCharge(cardToken, isRecurring);

                // Example - setup headers for connector API call
                var connRequest = new RestRequest(Method.POST);
                connRequest.AddHeader("content-type", Constants.JsonType);
                connRequest.AddHeader("Authorization", $"Bearer {configuration.SecretKey}");

                var settings = new JsonSerializerSettings()
                {
                    ContractResolver = new OrderedContractResolver(),
                    NullValueHandling = NullValueHandling.Ignore
                };

                // Example - Build json payload and Make API call
                // Check to see if a charge or an authorise by the CapturePayment flag
                if (request.CapturePayment == true)
                    connRestClient = new RestClient(Helpers.Environment.GetEnvironmentUrl(ChargeUrl.BaseProductionUrl, ChargeUrl.BaseSandboxUrl));
                else
                    connRestClient = new RestClient(Helpers.Environment.GetEnvironmentUrl(AuthoriseUrl.BaseProductionUrl, AuthoriseUrl.BaseSandboxUrl));

                var json = JsonConvert.SerializeObject(command, Formatting.Indented, settings);
                connRequest.AddParameter(Constants.JsonType, json, ParameterType.RequestBody);

                var connResponse = await connRestClient.ExecuteAsync(connRequest, cancellationToken);

                //Example - Handle response
                ChargeResponse resp =
                    JsonConvert.DeserializeObject<ChargeResponse>(connResponse.Content);

                return resp;
            }
            catch (Exception e)
            {
                // handle errors
                // may want to handle specific errors
                throw new GenericConnectorException(e.Message, "500");
            }
        }



        public class ProcessPaymentActionResolver : IProcessPaymentActionProvider
        {
            public IProcessPaymentAction GetHandler()
            {
                return new ProcessPaymentAction();
            }
        }
    }
}